#import "PrefController.h"
#import "GlobalConstants.h"

@implementation PrefController

- (id)init
{
    [super init];
    version=[NSMutableString stringWithCapacity:1];
    [version retain];
    return self;
}

- (void)dealloc
{
	[version release];
	[editor release];
	[latexEnginePath release];
	[ghostscriptEnginePath release];
	[super dealloc];
}

- (void)awakeFromNib
{
    [self setUpPreferences];
    [preferenceWindow setFrameUsingName:@"Preferences"];
    [preferenceWindow setFrameAutosaveName:@"Preferences"];
    if(![version isEqualToString:kDefaultVersionString] || (version == NULL))
    {
        // first time ever !!
        [self setFirstTimeDefaults];
    }

}

- (IBAction)cancelPrefs:(id)sender
{
    [preferenceWindow orderOut:nil];
}

- (IBAction)defaultPrefs:(id)sender
{
    [self setFirstTimeDefaults];
    [self setUpPreferences];
    [self setUpWindow];
}

- (IBAction)savePrefs:(id)sender
{
    NSUserDefaults	*defs=[NSUserDefaults standardUserDefaults];
    
    if([moveInsertionPointButton state]==NSOnState)
        [defs setBool:YES forKey:kTeXFoG_MoveInsertionPoint_pref];
    else
        [defs setBool:NO forKey:kTeXFoG_MoveInsertionPoint_pref];
    
    if([modifierKeyButton state]==NSOnState)
        [defs setBool:YES forKey:kTeXFoG_UseModifierKey_pref];
    else
        [defs setBool:NO forKey:kTeXFoG_UseModifierKey_pref];
    
    [defs setInteger:[modifierKeyPopUpButton indexOfSelectedItem] forKey:kTeXFoG_WhatModifierKey_pref];
	[defs setObject:[editorPopUpButton titleOfSelectedItem] forKey:kTeXFoG_Editor_pref];
	
	[defs setObject:[latexEngine stringValue] forKey:kTeXFoG_latex_pref];
	[defs setObject:[ghostscriptEngine stringValue] forKey:kTeXFoG_ghostscript_pref];
    
    [defs synchronize];
        
    [preferenceWindow orderOut:nil];
}

- (IBAction)showPrefs:(id)sender
{
    [self setUpPreferences];
    [self setUpWindow];
    [preferenceWindow makeKeyAndOrderFront:sender];
}

- (IBAction)toggleMoveInsertionPoint:(id)sender
{
    if([moveInsertionPointButton state]==NSOnState)
    {
        [modifierKeyButton setEnabled:YES];
        [modifierKeyPopUpButton setEnabled:YES];
    }
    else
    {
        [modifierKeyButton setEnabled:NO];
        [modifierKeyPopUpButton setEnabled:NO];
    }
}

- (void)setUpPreferences
{
    NSUserDefaults	*defs=[NSUserDefaults standardUserDefaults];
    
    if([defs objectForKey:kTeXFoG_Version_pref]!=nil)
    {
        [version setString:[defs objectForKey:kTeXFoG_Version_pref]];
        moveInsertionPoint=[defs boolForKey:kTeXFoG_MoveInsertionPoint_pref];
        useModifierKey=[defs boolForKey:kTeXFoG_UseModifierKey_pref];
        whatModifierKey=[defs integerForKey:kTeXFoG_WhatModifierKey_pref];
		if(editor)
			[editor release];
		if([defs objectForKey:kTeXFoG_Editor_pref])
			editor=[[NSString alloc] initWithString:[defs objectForKey:kTeXFoG_Editor_pref]];
		else
			editor=[[NSString alloc] initWithString:kDefaultEditor];
		
		if([defs objectForKey:kTeXFoG_latex_pref])
			latexEnginePath=[[NSString alloc] initWithString:[defs objectForKey:kTeXFoG_latex_pref]];
		else
			latexEnginePath=[[NSString alloc] initWithString:kDefaultLaTeX];
		
		if([defs objectForKey:kTeXFoG_ghostscript_pref])
			ghostscriptEnginePath=[[NSString alloc] initWithString:[defs objectForKey:kTeXFoG_ghostscript_pref]];
		else
			ghostscriptEnginePath=[[NSString alloc] initWithString:kDefaultGhostscript];
    }
}

- (void)setFirstTimeDefaults
{
    NSUserDefaults	*defs=[NSUserDefaults standardUserDefaults];
    
    [defs setObject:kDefaultVersionString forKey:kTeXFoG_Version_pref];
    [defs setBool:kDefaultMoveInsertionPoint forKey:kTeXFoG_MoveInsertionPoint_pref];
    [defs setBool:kDefaultUseModifierKey forKey:kTeXFoG_UseModifierKey_pref];
    [defs setInteger:kDefaultWhatModifierKey forKey:kTeXFoG_WhatModifierKey_pref];
	[defs setObject:kDefaultEditor forKey:kTeXFoG_Editor_pref];
	[defs setObject:kDefaultLaTeX forKey:kTeXFoG_latex_pref];
	[defs setObject:kDefaultGhostscript forKey:kTeXFoG_ghostscript_pref];
    [defs synchronize];
}

- (void)setUpWindow
{
    if(moveInsertionPoint)
        [moveInsertionPointButton setState:NSOnState];
    else
        [moveInsertionPointButton setState:NSOffState];
    
    if(useModifierKey)
        [modifierKeyButton setState:NSOnState];
    else
        [modifierKeyButton setState:NSOffState];
    
    [modifierKeyPopUpButton selectItemAtIndex:whatModifierKey];
    [self toggleMoveInsertionPoint:self];
	[editorPopUpButton selectItemWithTitle:editor];
	[latexEngine setStringValue:latexEnginePath];
	[ghostscriptEngine setStringValue:ghostscriptEnginePath];
}

@end
